@@ -9,7 +9,6 @@
 
 #include "nsCOMPtr.h"
 #include "nsICertificateDialogs.h"
-#include "nsIClientAuthDialogs.h"
 #include "nsIStringBundle.h"
 #include "nsITokenPasswordDialogs.h"
 
@@ -21,13 +20,11 @@
   }
 
 class nsNSSDialogs : public nsICertificateDialogs,
-                     public nsIClientAuthDialogs,
                      public nsITokenPasswordDialogs {
  public:
   NS_DECL_THREADSAFE_ISUPPORTS
   NS_DECL_NSITOKENPASSWORDDIALOGS
   NS_DECL_NSICERTIFICATEDIALOGS
-  NS_DECL_NSICLIENTAUTHDIALOGS
   nsNSSDialogs();
 
   nsresult Init();
