@@ -73,7 +73,7 @@ function openSetP12PasswordDialog() {
     "",
     returnVals
   );
-  return new Promise((resolve, reject) => {
+  return new Promise(resolve => {
     win.addEventListener(
       "load",
       function () {
