@@ -141,7 +141,7 @@ decorate_task(
   withStub(Normandy, "finishInit"),
   async function testStartupDelayed({ finishInitStub }) {
     let originalDeferred = Normandy.uiAvailableNotificationObserved;
-    let mockUiAvailableDeferred = PromiseUtils.defer();
+    let mockUiAvailableDeferred = Promise.withResolvers();
     Normandy.uiAvailableNotificationObserved = mockUiAvailableDeferred;
 
     let initPromise = Normandy.init();
@@ -304,83 +304,3 @@ decorate_task(
     ok(PreferenceRollouts.init.called, "startup calls PreferenceRollouts.init");
   }
 );
-
-// Test that disabling telemetry removes all stored enrollment IDs
-decorate_task(
-  PreferenceExperiments.withMockExperiments([
-    factories.preferenceStudyFactory({
-      enrollmentId: "test-enrollment-id",
-    }),
-  ]),
-  AddonStudies.withStudies([
-    factories.addonStudyFactory({ slug: "test-study" }),
-  ]),
-  PreferenceRollouts.withTestMock(),
-  AddonRollouts.withTestMock(),
-  async function disablingTelemetryClearsEnrollmentIds({
-    prefExperiments: [prefExperiment],
-    addonStudies: [addonStudy],
-  }) {
-    const prefRollout = {
-      slug: "test-rollout",
-      state: PreferenceRollouts.STATE_ACTIVE,
-      preferences: [],
-      enrollmentId: "test-enrollment-id",
-    };
-    await PreferenceRollouts.add(prefRollout);
-    const addonRollout = {
-      slug: "test-rollout",
-      state: AddonRollouts.STATE_ACTIVE,
-      extension: {},
-      enrollmentId: "test-enrollment-id",
-    };
-    await AddonRollouts.add(addonRollout);
-
-    // pre-check
-    ok(
-      (await PreferenceExperiments.get(prefExperiment.slug)).enrollmentId,
-      "pref experiment should have an enrollment id"
-    );
-    ok(
-      (await AddonStudies.get(addonStudy.recipeId)).enrollmentId,
-      "addon study should have an enrollment id"
-    );
-    ok(
-      (await PreferenceRollouts.get(prefRollout.slug)).enrollmentId,
-      "pref rollout should have an enrollment id"
-    );
-    ok(
-      (await AddonRollouts.get(addonRollout.slug)).enrollmentId,
-      "addon rollout should have an enrollment id"
-    );
-
-    // trigger telemetry being disabled
-    await Normandy.observe(
-      null,
-      TelemetryUtils.TELEMETRY_UPLOAD_DISABLED_TOPIC,
-      null
-    );
-
-    // no enrollment IDs anymore
-    is(
-      (await PreferenceExperiments.get(prefExperiment.slug)).enrollmentId,
-      TelemetryEvents.NO_ENROLLMENT_ID_MARKER,
-      "pref experiment should not have an enrollment id"
-    );
-    is(
-      (await AddonStudies.get(addonStudy.recipeId)).enrollmentId,
-      TelemetryEvents.NO_ENROLLMENT_ID_MARKER,
-      "addon study should not have an enrollment id"
-    );
-    is(
-      (await PreferenceRollouts.get(prefRollout.slug)).enrollmentId,
-      TelemetryEvents.NO_ENROLLMENT_ID_MARKER,
-      "pref rollout should not have an enrollment id"
-    );
-    is(
-      (await AddonRollouts.get(addonRollout.slug)).enrollmentId,
-      TelemetryEvents.NO_ENROLLMENT_ID_MARKER,
-      "addon rollout should not have an enrollment id"
-    );
-  }
-);
