@@ -57,6 +57,8 @@
 #define TAGS_ROOT_GUID "tags________"
 #define MOBILE_ROOT_GUID "mobile______"
 
+#define SQL_QUOTE(text) "'" text "'"
+
 class mozIStorageValueArray;
 class nsIAutoCompleteController;
 class nsIEffectiveTLDService;
@@ -184,6 +186,9 @@ class nsNavHistory final : public nsSupportsWeakReference,
   static const int32_t kGetInfoIndex_VisitId;
   static const int32_t kGetInfoIndex_FromVisitId;
   static const int32_t kGetInfoIndex_VisitType;
+  static const int32_t kGetTargetFolder_Guid;
+  static const int32_t kGetTargetFolder_ItemId;
+  static const int32_t kGetTargetFolder_Title;
 
   int64_t GetTagsFolder();
 
@@ -199,21 +204,16 @@ class nsNavHistory final : public nsSupportsWeakReference,
   nsresult RowToResult(mozIStorageValueArray* aRow,
                        nsNavHistoryQueryOptions* aOptions,
                        nsNavHistoryResultNode** aResult);
-  nsresult QueryRowToResult(int64_t aItemId, const nsACString& aBookmarkGuid,
-                            const nsACString& aURI, const nsACString& aTitle,
+
+  nsresult QueryUriToResult(const nsACString& aQueryURI, int64_t aItemId,
+                            const nsACString& aBookmarkGuid,
+                            const nsACString& aTitle,
+                            int64_t aTargetFolderItemId,
+                            const nsACString& aTargetFolderGuid,
+                            const nsACString& aTargetFolderTitle,
                             uint32_t aAccessCount, PRTime aTime,
                             nsNavHistoryResultNode** aNode);
 
-  nsresult VisitIdToResultNode(int64_t visitId,
-                               nsNavHistoryQueryOptions* aOptions,
-                               nsNavHistoryResultNode** aResult);
-
-  nsresult BookmarkIdToResultNode(int64_t aBookmarkId,
-                                  nsNavHistoryQueryOptions* aOptions,
-                                  nsNavHistoryResultNode** aResult);
-  nsresult URIToResultNode(nsIURI* aURI, nsNavHistoryQueryOptions* aOptions,
-                           nsNavHistoryResultNode** aResult);
-
   /**
    * Returns current number of days stored in history.
    */
@@ -309,6 +309,20 @@ class nsNavHistory final : public nsSupportsWeakReference,
    */
   void UpdateDaysOfHistory(PRTime visitTime);
 
+  /**
+   * Get a SQL fragment to pre-cache all the tagged bookmark into a `tagged`
+   * CTE.
+   */
+  static nsLiteralCString GetTagsSqlFragment(const uint16_t aQueryType,
+                                             bool aExcludeItems);
+
+  /**
+   * Get target folder guid from given query URI.
+   * If the folder guid is not found, returns Nonthing().
+   */
+  static mozilla::Maybe<nsCString> GetTargetFolderGuid(
+      const nsACString& aQueryURI);
+
   /**
    * Store last insterted id for a table.
    */
@@ -336,6 +350,10 @@ class nsNavHistory final : public nsSupportsWeakReference,
 
   static void InvalidateDaysOfHistory();
 
+  static nsresult TokensToQuery(
+      const nsTArray<mozilla::places::QueryKeyValuePair>& aTokens,
+      nsNavHistoryQuery* aQuery, nsNavHistoryQueryOptions* aOptions);
+
  private:
   ~nsNavHistory();
 
@@ -436,15 +454,7 @@ class nsNavHistory final : public nsSupportsWeakReference,
   int32_t mUnvisitedTypedBonus;
   int32_t mReloadVisitBonus;
 
-  nsresult RecalculateOriginFrecencyStatsInternal();
-
-  // in nsNavHistoryQuery.cpp
-  nsresult TokensToQuery(
-      const nsTArray<mozilla::places::QueryKeyValuePair>& aTokens,
-      nsNavHistoryQuery* aQuery, nsNavHistoryQueryOptions* aOptions);
-
   int64_t mTagsFolder;
-
   int64_t mLastCachedStartOfDay;
   int64_t mLastCachedEndOfDay;
 };
@@ -452,7 +462,7 @@ class nsNavHistory final : public nsSupportsWeakReference,
 #define PLACES_URI_PREFIX "place:"
 
 /* Returns true if the given URI represents a history query. */
-inline bool IsQueryURI(const nsCString& uri) {
+inline static bool IsQueryURI(const nsACString& uri) {
   return StringBeginsWith(uri, nsLiteralCString(PLACES_URI_PREFIX));
 }
 
