@@ -246,7 +246,7 @@ add_task(async function test_import_chromefavicon() {
   let data = await new Promise(resolve => {
     PlacesUtils.favicons.getFaviconDataForPage(
       PAGE_URI,
-      (uri, dataLen, faviconData, mimeType) => resolve(faviconData)
+      (uri, dataLen, faviconData) => resolve(faviconData)
     );
   });
 
@@ -366,13 +366,14 @@ function checkItem(aExpected, aNode) {
         case "url":
           Assert.equal(aNode.uri, aExpected.url);
           break;
-        case "icon":
+        case "icon": {
           let { data } = await getFaviconDataForPage(aExpected.url);
           let base64Icon =
             "data:image/png;base64," +
             base64EncodeString(String.fromCharCode.apply(String, data));
           Assert.ok(base64Icon == aExpected.icon);
           break;
+        }
         case "keyword": {
           let entry = await PlacesUtils.keywords.fetch({ url: aNode.uri });
           Assert.equal(entry.keyword, aExpected.keyword);
@@ -383,7 +384,7 @@ function checkItem(aExpected, aNode) {
           Assert.equal(entry.postData, aExpected.postData);
           break;
         }
-        case "charset":
+        case "charset": {
           let pageInfo = await PlacesUtils.history.fetch(aNode.uri, {
             includeAnnotations: true,
           });
@@ -392,10 +393,11 @@ function checkItem(aExpected, aNode) {
             aExpected.charset
           );
           break;
+        }
         case "feedUrl":
           // No more supported.
           break;
-        case "children":
+        case "children": {
           let folder = aNode.QueryInterface(
             Ci.nsINavHistoryContainerResultNode
           );
@@ -409,6 +411,7 @@ function checkItem(aExpected, aNode) {
 
           folder.containerOpen = false;
           break;
+        }
         default:
           throw new Error("Unknown property");
       }
