@@ -4,7 +4,7 @@
 
 // This file defines these globals on the window object.
 // Define them here so that ESLint can find them:
-/* globals MozXULElement, MozElements */
+/* globals MozXULElement, MozHTMLElement, MozElements */
 
 "use strict";
 
@@ -21,16 +21,10 @@
   const MozElements = {};
   window.MozElements = MozElements;
 
-  const { Services } = ChromeUtils.import(
-    "resource://gre/modules/Services.jsm"
+  const { AppConstants } = ChromeUtils.importESModule(
+    "resource://gre/modules/AppConstants.sys.mjs"
   );
-  const { AppConstants } = ChromeUtils.import(
-    "resource://gre/modules/AppConstants.jsm"
-  );
-  const env = Cc["@mozilla.org/process/environment;1"].getService(
-    Ci.nsIEnvironment
-  );
-  const instrumentClasses = env.get("MOZ_INSTRUMENT_CUSTOM_ELEMENTS");
+  const instrumentClasses = Services.env.get("MOZ_INSTRUMENT_CUSTOM_ELEMENTS");
   const instrumentedClasses = instrumentClasses ? new Set() : null;
   const instrumentedBaseClasses = instrumentClasses ? new WeakSet() : null;
 
@@ -38,7 +32,7 @@
   // to modify the class so we can instrument function calls in local development:
   if (instrumentClasses) {
     let define = window.customElements.define;
-    window.customElements.define = function(name, c, opts) {
+    window.customElements.define = function (name, c, opts) {
       instrumentCustomElementClass(c);
       return define.call(this, name, c, opts);
     };
@@ -51,7 +45,7 @@
     );
   }
 
-  MozElements.printInstrumentation = function(collapsed) {
+  MozElements.printInstrumentation = function (collapsed) {
     let summaries = [];
     let totalCalls = 0;
     let totalTime = 0;
@@ -70,9 +64,7 @@
       }
     }
     if (summaries.length) {
-      let groupName = `Instrumentation data for custom elements in ${
-        document.documentURI
-      }`;
+      let groupName = `Instrumentation data for custom elements in ${document.documentURI}`;
       console[collapsed ? "groupCollapsed" : "group"](groupName);
       console.log(
         `Total function calls ${totalCalls} and total time spent inside ${totalTime.toFixed(
@@ -121,7 +113,7 @@
     let data = { instances: 0 };
 
     function wrapFunction(name, fn) {
-      return function() {
+      return function () {
         if (!data[name]) {
           data[name] = { time: 0, calls: 0 };
         }
@@ -372,8 +364,8 @@
 
         for (let [selector, newAttr] of list) {
           if (!(selector in this._inheritedElements)) {
-            let parent = this.shadowRoot || this;
-            this._inheritedElements[selector] = parent.querySelector(selector);
+            this._inheritedElements[selector] =
+              this.getElementForAttrInheritance(selector);
           }
           let el = this._inheritedElements[selector];
           if (el) {
@@ -388,6 +380,51 @@
         }
       }
 
+      /**
+       * Used in setting up attribute inheritance. Takes a selector and returns
+       * an element for that selector from shadow DOM if there is a shadowRoot,
+       * or from the light DOM if not.
+       *
+       * Here's one problem this solves. ElementB extends ElementA which extends
+       * MozXULElement. ElementA has a shadowRoot. ElementB tries to inherit
+       * attributes in light DOM by calling `initializeAttributeInheritance`
+       * but that fails because it defaults to inheriting from the shadow DOM
+       * and not the light DOM. (See bug 1545824.)
+       *
+       * To solve this, ElementB can override `getElementForAttrInheritance` so
+       * it queries the light DOM for some selectors as needed. For example:
+       *
+       *  class ElementA extends MozXULElement {
+       *    static get inheritedAttributes() {
+       *      return { ".one": "attr" };
+       *    }
+       *  }
+       *
+       *  class ElementB extends customElements.get("elementa") {
+       *    static get inheritedAttributes() {
+       *      return Object.assign({}, super.inheritedAttributes(), {
+       *        ".two": "attr",
+       *      });
+       *    }
+       *    getElementForAttrInheritance(selector) {
+       *      if (selector == ".two") {
+       *        return this.querySelector(selector)
+       *      } else {
+       *        return super.getElementForAttrInheritance(selector);
+       *      }
+       *    }
+       *  }
+       *
+       * @param {string} selector
+       *        A selector used to query an element.
+       *
+       * @return {Element} The element found by the selector.
+       */
+      getElementForAttrInheritance(selector) {
+        let parent = this.shadowRoot || this;
+        return parent.querySelector(selector);
+      }
+
       /**
        * Sometimes an element may not want to run connectedCallback logic during
        * parse. This could be because we don't want to initialize the element before
@@ -413,6 +450,55 @@
         return gIsDOMContentLoaded && this.isConnected;
       }
 
+      /**
+       * Passes DOM events to the on_<event type> methods.
+       */
+      handleEvent(event) {
+        let methodName = "on_" + event.type;
+        if (methodName in this) {
+          this[methodName](event);
+        } else {
+          throw new Error("Unrecognized event: " + event.type);
+        }
+      }
+
+      /**
+       * Used by custom elements for caching fragments. We now would be
+       * caching once per class while also supporting subclasses.
+       *
+       * If available, returns the cached fragment.
+       * Otherwise, creates it.
+       *
+       * Example:
+       *
+       *  class ElementA extends MozXULElement {
+       *    static get markup() {
+       *      return `<hbox class="example"`;
+       *    }
+       *
+       *    connectedCallback() {
+       *      this.appendChild(this.constructor.fragment);
+       *    }
+       *  }
+       *
+       * @return {importedNode} The imported node that has not been
+       * inserted into document tree.
+       */
+      static get fragment() {
+        if (!this.hasOwnProperty("_fragment")) {
+          let markup = this.markup;
+          if (markup) {
+            this._fragment = MozXULElement.parseXULToFragment(
+              markup,
+              this.entities
+            );
+          } else {
+            throw new Error("Markup is null");
+          }
+        }
+        return document.importNode(this._fragment, true);
+      }
+
       /**
        * Allows eager deterministic construction of XUL elements with XBL attached, by
        * parsing an element tree and returning a DOM fragment to be inserted in the
@@ -440,7 +526,7 @@
        *         but excluding any text node.
        */
       static parseXULToFragment(str, entities = []) {
-        let doc = gXULDOMParser.parseFromString(
+        let doc = gXULDOMParser.parseFromSafeString(
           `
       ${
         entities.length
@@ -463,6 +549,11 @@
     `,
           "application/xml"
         );
+
+        if (doc.documentElement.localName === "parsererror") {
+          throw new Error("not well-formed XML");
+        }
+
         // The XUL/XBL parser is set to ignore all-whitespace nodes, whereas (X)HTML
         // does not do this. Most XUL code assumes that the whitespace has been
         // stripped out, so we simply remove all text nodes after using the parser.
@@ -496,7 +587,10 @@
       static insertFTLIfNeeded(path) {
         let container = document.head || document.querySelector("linkset");
         if (!container) {
-          if (document.contentType == "application/vnd.mozilla.xul+xml") {
+          if (
+            document.documentElement.namespaceURI ===
+            "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul"
+          ) {
             container = document.createXULElement("linkset");
             document.documentElement.appendChild(container);
           } else if (document.documentURI == AppConstants.BROWSER_CHROME_URL) {
@@ -546,18 +640,17 @@
         cls.prototype.customInterfaces = ifaces;
 
         cls.prototype.QueryInterface = ChromeUtils.generateQI(ifaces);
-        cls.prototype.getCustomInterfaceCallback = function getCustomInterfaceCallback(
-          ifaceToCheck
-        ) {
-          if (
-            cls.prototype.customInterfaces.some(iface =>
-              iface.equals(ifaceToCheck)
-            )
-          ) {
-            return getInterfaceProxy(this);
-          }
-          return null;
-        };
+        cls.prototype.getCustomInterfaceCallback =
+          function getCustomInterfaceCallback(ifaceToCheck) {
+            if (
+              cls.prototype.customInterfaces.some(iface =>
+                iface.equals(ifaceToCheck)
+              )
+            ) {
+              return getInterfaceProxy(this);
+            }
+            return null;
+          };
       }
     };
 
@@ -570,6 +663,7 @@
   };
 
   const MozXULElement = MozElements.MozElementMixin(XULElement);
+  const MozHTMLElement = MozElements.MozElementMixin(HTMLElement);
 
   /**
    * Given an object, add a proxy that reflects interface implementations
@@ -582,10 +676,10 @@
         get(target, prop, receiver) {
           let propOrMethod = target[prop];
           if (typeof propOrMethod == "function") {
-            if (propOrMethod instanceof MozQueryInterface) {
+            if (MozQueryInterface.isInstance(propOrMethod)) {
               return Reflect.get(target, prop, receiver);
             }
-            return function(...args) {
+            return function (...args) {
               return propOrMethod.apply(target, args);
             };
           }
@@ -635,25 +729,14 @@
     class BaseText extends MozElements.BaseControlMixin(Base) {
       set label(val) {
         this.setAttribute("label", val);
-        return val;
       }
 
       get label() {
-        return this.getAttribute("label");
-      }
-
-      set crop(val) {
-        this.setAttribute("crop", val);
-        return val;
-      }
-
-      get crop() {
-        return this.getAttribute("crop");
+        return this.getAttribute("label") || "";
       }
 
       set image(val) {
         this.setAttribute("image", val);
-        return val;
       }
 
       get image() {
@@ -662,7 +745,6 @@
 
       set command(val) {
         this.setAttribute("command", val);
-        return val;
       }
 
       get command() {
@@ -677,13 +759,10 @@
         if (this.labelElement) {
           this.labelElement.accessKey = val;
         }
-        return val;
       }
 
       get accessKey() {
-        return this.labelElement
-          ? this.labelElement.accessKey
-          : this.getAttribute("accesskey");
+        return this.labelElement?.accessKey || this.getAttribute("accesskey");
       }
     };
   MozElements.BaseTextMixin = BaseTextMixin;
@@ -691,6 +770,7 @@
 
   // Attach the base class to the window so other scripts can use it:
   window.MozXULElement = MozXULElement;
+  window.MozHTMLElement = MozHTMLElement;
 
   customElements.setElementCreationCallback("browser", () => {
     Services.scriptloader.loadSubScript(
@@ -699,40 +779,30 @@
     );
   });
 
-  // For now, don't load any elements in the extension dummy document.
-  // We will want to load <browser> when that's migrated (bug 1441935).
-  const isDummyDocument =
-    document.documentURI == "chrome://extensions/content/dummy.xul";
-  if (!isDummyDocument) {
-    for (let script of [
-      "chrome://global/content/elements/general.js",
-      "chrome://global/content/elements/button.js",
-      "chrome://global/content/elements/checkbox.js",
-      "chrome://global/content/elements/menu.js",
-      "chrome://global/content/elements/menupopup.js",
-      "chrome://global/content/elements/notificationbox.js",
-      "chrome://global/content/elements/popupnotification.js",
-      "chrome://global/content/elements/radio.js",
-      "chrome://global/content/elements/richlistbox.js",
-      "chrome://global/content/elements/autocomplete-popup.js",
-      "chrome://global/content/elements/autocomplete-richlistitem.js",
-      "chrome://global/content/elements/textbox.js",
-      "chrome://global/content/elements/tabbox.js",
-      "chrome://global/content/elements/text.js",
-      "chrome://global/content/elements/tree.js",
-      "chrome://global/content/elements/wizard.js",
-    ]) {
-      Services.scriptloader.loadSubScript(script, window);
-    }
-
+  // Skip loading any extra custom elements in the extension dummy document
+  // and GeckoView windows.
+  const loadExtraCustomElements = !(
+    document.documentURI == "chrome://extensions/content/dummy.xhtml" ||
+    document.documentURI == "chrome://geckoview/content/geckoview.xhtml"
+  );
+  if (loadExtraCustomElements) {
+    // Lazily load the following elements
     for (let [tag, script] of [
+      ["button-group", "chrome://global/content/elements/named-deck.js"],
       ["findbar", "chrome://global/content/elements/findbar.js"],
       ["menulist", "chrome://global/content/elements/menulist.js"],
+      ["named-deck", "chrome://global/content/elements/named-deck.js"],
+      ["named-deck-button", "chrome://global/content/elements/named-deck.js"],
+      ["panel-list", "chrome://global/content/elements/panel-list.js"],
       ["search-textbox", "chrome://global/content/elements/search-textbox.js"],
       ["stringbundle", "chrome://global/content/elements/stringbundle.js"],
       [
-        "printpreview-toolbar",
-        "chrome://global/content/printPreviewToolbar.js",
+        "printpreview-pagination",
+        "chrome://global/content/printPreviewPagination.js",
+      ],
+      [
+        "autocomplete-input",
+        "chrome://global/content/elements/autocomplete-input.js",
       ],
       ["editor", "chrome://global/content/elements/editor.js"],
     ]) {
@@ -740,5 +810,80 @@
         Services.scriptloader.loadSubScript(script, window);
       });
     }
+
+    document.addEventListener(
+      "DOMContentLoaded",
+      () => {
+        // Only sync-import widgets once the document has loaded. If a widget is
+        // used before DOMContentLoaded it will be imported and upgraded when
+        // registering the customElements.setElementCreationCallback().
+        for (let [tag, script] of [
+          ["moz-button", "chrome://global/content/elements/moz-button.mjs"],
+          [
+            "moz-button-group",
+            "chrome://global/content/elements/moz-button-group.mjs",
+          ],
+          ["moz-card", "chrome://global/content/elements/moz-card.mjs"],
+          ["moz-checkbox", "chrome://global/content/elements/moz-checkbox.mjs"],
+          ["moz-fieldset", "chrome://global/content/elements/moz-fieldset.mjs"],
+          [
+            "moz-five-star",
+            "chrome://global/content/elements/moz-five-star.mjs",
+          ],
+          ["moz-label", "chrome://global/content/elements/moz-label.mjs"],
+          [
+            "moz-message-bar",
+            "chrome://global/content/elements/moz-message-bar.mjs",
+          ],
+          ["moz-page-nav", "chrome://global/content/elements/moz-page-nav.mjs"],
+          ["moz-radio", "chrome://global/content/elements/moz-radio-group.mjs"],
+          [
+            "moz-radio-group",
+            "chrome://global/content/elements/moz-radio-group.mjs",
+          ],
+          [
+            "moz-support-link",
+            "chrome://global/content/elements/moz-support-link.mjs",
+          ],
+          ["moz-toggle", "chrome://global/content/elements/moz-toggle.mjs"],
+        ]) {
+          if (!customElements.get(tag)) {
+            customElements.setElementCreationCallback(
+              tag,
+              function customElementCreationCallback() {
+                ChromeUtils.importESModule(script, { global: "current" });
+              }
+            );
+          }
+        }
+      },
+      { once: true }
+    );
+
+    // Immediately load the following elements
+    for (let script of [
+      "chrome://global/content/elements/arrowscrollbox.js",
+      "chrome://global/content/elements/dialog.js",
+      "chrome://global/content/elements/general.js",
+      "chrome://global/content/elements/button.js",
+      "chrome://global/content/elements/checkbox.js",
+      "chrome://global/content/elements/menu.js",
+      "chrome://global/content/elements/menupopup.js",
+      "chrome://global/content/elements/moz-input-box.js",
+      "chrome://global/content/elements/notificationbox.js",
+      "chrome://global/content/elements/panel.js",
+      "chrome://global/content/elements/popupnotification.js",
+      "chrome://global/content/elements/radio.js",
+      "chrome://global/content/elements/richlistbox.js",
+      "chrome://global/content/elements/autocomplete-popup.js",
+      "chrome://global/content/elements/autocomplete-richlistitem.js",
+      "chrome://global/content/elements/tabbox.js",
+      "chrome://global/content/elements/text.js",
+      "chrome://global/content/elements/toolbarbutton.js",
+      "chrome://global/content/elements/tree.js",
+      "chrome://global/content/elements/wizard.js",
+    ]) {
+      Services.scriptloader.loadSubScript(script, window);
+    }
   }
 })();
