@@ -6,25 +6,30 @@
 
 // We attach Preferences to the window object so other contexts (tests, JSMs)
 // have access to it.
-const Preferences = window.Preferences = (function() {
-  ChromeUtils.import("resource://gre/modules/EventEmitter.jsm");
-  ChromeUtils.import("resource://gre/modules/Services.jsm");
-  ChromeUtils.import("resource://gre/modules/XPCOMUtils.jsm");
+const Preferences = (window.Preferences = (function () {
+  const { EventEmitter } = ChromeUtils.importESModule(
+    "resource://gre/modules/EventEmitter.sys.mjs"
+  );
 
   const lazy = {};
-  ChromeUtils.defineModuleGetter(lazy, "DeferredTask",
-                                 "resource://gre/modules/DeferredTask.jsm");
+  ChromeUtils.defineESModuleGetters(lazy, {
+    DeferredTask: "resource://gre/modules/DeferredTask.sys.mjs",
+  });
 
   function getElementsByAttribute(name, value) {
     // If we needed to defend against arbitrary values, we would escape
     // double quotes (") and escape characters (\) in them, i.e.:
     //   ${value.replace(/["\\]/g, '\\$&')}
-    return value ? document.querySelectorAll(`[${name}="${value}"]`)
-                 : document.querySelectorAll(`[${name}]`);
+    return value
+      ? document.querySelectorAll(`[${name}="${value}"]`)
+      : document.querySelectorAll(`[${name}]`);
   }
 
   const domContentLoadedPromise = new Promise(resolve => {
-    window.addEventListener("DOMContentLoaded", resolve, { capture: true, once: true });
+    window.addEventListener("DOMContentLoaded", resolve, {
+      capture: true,
+      once: true,
+    });
   });
 
   const Preferences = {
@@ -36,7 +41,11 @@ const Preferences = window.Preferences = (function() {
       }
       const pref = new Preference(prefInfo);
       this._all[pref.id] = pref;
-      domContentLoadedPromise.then(() => { pref.updateElements(); });
+      domContentLoadedPromise.then(() => {
+        if (!this.updateQueued) {
+          pref.updateElements();
+        }
+      });
       return pref;
     },
 
@@ -65,21 +74,14 @@ const Preferences = window.Preferences = (function() {
 
     get instantApply() {
       // The about:preferences page forces instantApply.
+      // TODO: Remove forceEnableInstantApply in favor of always applying in a
+      // parent and never applying in a child (bug 1775386).
       if (this._instantApplyForceEnabled) {
         return true;
       }
 
       // Dialogs of type="child" are never instantApply.
-      if (this.type === "child") {
-        return false;
-      }
-
-      // All other pref windows observe the value of the instantApply
-      // preference.  Note that, as of this writing, the only such windows
-      // are in tests, so it should be possible to remove the pref
-      // (and forceEnableInstantApply) in favor of always applying in a parent
-      // and never applying in a child.
-      return Services.prefs.getBoolPref("browser.preferences.instantApply");
+      return this.type !== "child";
     },
 
     _instantApplyForceEnabled: false,
@@ -96,33 +98,45 @@ const Preferences = window.Preferences = (function() {
       }
     },
 
-    onDOMContentLoaded() {
-      // Iterate elements with a "preference" attribute and log an error
-      // if there isn't a corresponding Preference object in order to catch
-      // any cases of elements referencing preferences that haven't (yet?)
-      // been registered.
-      //
-      // TODO: remove this code once we determine that there are no such
-      // elements (or resolve any bugs that cause this behavior).
-      //
-      const elements = getElementsByAttribute("preference");
-      for (const element of elements) {
-        const id = element.getAttribute("preference");
-        const pref = this.get(id);
-        if (!pref) {
-          console.error(`Missing preference for ID ${id}`);
-        }
+    updateQueued: false,
+
+    queueUpdateOfAllElements() {
+      if (this.updateQueued) {
+        return;
       }
+
+      this.updateQueued = true;
+
+      Services.tm.dispatchToMainThread(() => {
+        let startTime = performance.now();
+
+        const elements = getElementsByAttribute("preference");
+        for (const element of elements) {
+          const id = element.getAttribute("preference");
+          let preference = this.get(id);
+          if (!preference) {
+            console.error(`Missing preference for ID ${id}`);
+            continue;
+          }
+
+          preference.setElementValue(element);
+        }
+
+        ChromeUtils.addProfilerMarker(
+          "Preferences",
+          { startTime },
+          `updateAllElements: ${elements.length} preferences updated`
+        );
+
+        this.updateQueued = false;
+      });
     },
 
     onUnload() {
       Services.prefs.removeObserver("", this);
     },
 
-    QueryInterface: XPCOMUtils.generateQI([
-      Ci.nsITimerCallback,
-      Ci.nsIObserver,
-    ]),
+    QueryInterface: ChromeUtils.generateQI(["nsITimerCallback", "nsIObserver"]),
 
     _deferredValueUpdateElements: new Set(),
 
@@ -145,11 +159,14 @@ const Preferences = window.Preferences = (function() {
 
     getPreferenceElement(aStartElement) {
       let temp = aStartElement;
-      while (temp && temp.nodeType == Node.ELEMENT_NODE &&
-             !temp.hasAttribute("preference"))
+      while (
+        temp &&
+        temp.nodeType == Node.ELEMENT_NODE &&
+        !temp.hasAttribute("preference")
+      ) {
         temp = temp.parentNode;
-      return temp && temp.nodeType == Node.ELEMENT_NODE ?
-             temp : aStartElement;
+      }
+      return temp && temp.nodeType == Node.ELEMENT_NODE ? temp : aStartElement;
     },
 
     _deferredValueUpdate(aElement) {
@@ -173,13 +190,17 @@ const Preferences = window.Preferences = (function() {
       const element = this.getPreferenceElement(aElement);
       if (element.hasAttribute("preference")) {
         if (element.getAttribute("delayprefsave") != "true") {
-          const preference = Preferences.get(element.getAttribute("preference"));
+          const preference = Preferences.get(
+            element.getAttribute("preference")
+          );
           const prefVal = preference.getElementValue(element);
           preference.value = prefVal;
         } else {
           if (!element._deferredValueUpdateTask) {
-            element._deferredValueUpdateTask =
-              new lazy.DeferredTask(this._deferredValueUpdate.bind(this, element), 1000);
+            element._deferredValueUpdateTask = new lazy.DeferredTask(
+              this._deferredValueUpdate.bind(this, element),
+              1000
+            );
             this._deferredValueUpdateElements.add(element);
           } else {
             // Each time the preference is changed, restart the delay.
@@ -193,18 +214,12 @@ const Preferences = window.Preferences = (function() {
     onCommand(event) {
       // This "command" event handler tracks changes made to preferences by
       // the user in this window.
-      if (event.sourceEvent)
+      if (event.sourceEvent) {
         event = event.sourceEvent;
+      }
       this.userChangedValue(event.target);
     },
 
-    onSelect(event) {
-      // This "select" event handler tracks changes made to colorpicker
-      // preferences by the user in this window.
-      if (event.target.localName == "colorpicker")
-        this.userChangedValue(event.target);
-    },
-
     onChange(event) {
       // This "change" event handler tracks changes made to preferences by
       // the user in this window.
@@ -218,26 +233,21 @@ const Preferences = window.Preferences = (function() {
     },
 
     _fireEvent(aEventName, aTarget) {
-      // Panel loaded, synthesize a load event.
       try {
-        const event = document.createEvent("Events");
-        event.initEvent(aEventName, true, true);
-        let cancel = !aTarget.dispatchEvent(event);
-        if (aTarget.hasAttribute("on" + aEventName)) {
-          const fn = new Function("event", aTarget.getAttribute("on" + aEventName));
-          const rv = fn.call(aTarget, event);
-          if (!rv)
-            cancel = true;
-        }
-        return !cancel;
+        const event = new CustomEvent(aEventName, {
+          bubbles: true,
+          cancelable: true,
+        });
+        return aTarget.dispatchEvent(event);
       } catch (e) {
-        Cu.reportError(e);
+        console.error(e);
       }
       return false;
     },
 
     onDialogAccept(event) {
-      if (!this._fireEvent("beforeaccept", document.documentElement)) {
+      let dialog = document.querySelector("dialog");
+      if (!this._fireEvent("beforeaccept", dialog)) {
         event.preventDefault();
         return false;
       }
@@ -246,27 +256,75 @@ const Preferences = window.Preferences = (function() {
     },
 
     close(event) {
-      if (Preferences.instantApply)
+      if (Preferences.instantApply) {
         window.close();
+      }
       event.stopPropagation();
       event.preventDefault();
     },
 
     handleEvent(event) {
       switch (event.type) {
-        case "change": return this.onChange(event);
-        case "command": return this.onCommand(event);
-        case "dialogaccept": return this.onDialogAccept(event);
-        case "input": return this.onInput(event);
-        case "select": return this.onSelect(event);
-        case "unload": return this.onUnload(event);
-        default: return undefined;
+        case "toggle":
+        case "change":
+          return this.onChange(event);
+        case "command":
+          return this.onCommand(event);
+        case "dialogaccept":
+          return this.onDialogAccept(event);
+        case "input":
+          return this.onInput(event);
+        case "unload":
+          return this.onUnload(event);
+        default:
+          return undefined;
       }
     },
+
+    _syncFromPrefListeners: new WeakMap(),
+    _syncToPrefListeners: new WeakMap(),
+
+    addSyncFromPrefListener(aElement, callback) {
+      this._syncFromPrefListeners.set(aElement, callback);
+      if (this.updateQueued) {
+        return;
+      }
+      // Make sure elements are updated correctly with the listener attached.
+      let elementPref = aElement.getAttribute("preference");
+      if (elementPref) {
+        let pref = this.get(elementPref);
+        if (pref) {
+          pref.updateElements();
+        }
+      }
+    },
+
+    addSyncToPrefListener(aElement, callback) {
+      this._syncToPrefListeners.set(aElement, callback);
+      if (this.updateQueued) {
+        return;
+      }
+      // Make sure elements are updated correctly with the listener attached.
+      let elementPref = aElement.getAttribute("preference");
+      if (elementPref) {
+        let pref = this.get(elementPref);
+        if (pref) {
+          pref.updateElements();
+        }
+      }
+    },
+
+    removeSyncFromPrefListener(aElement) {
+      this._syncFromPrefListeners.delete(aElement);
+    },
+
+    removeSyncToPrefListener(aElement) {
+      this._syncToPrefListeners.delete(aElement);
+    },
   };
 
   Services.prefs.addObserver("", Preferences);
-  domContentLoadedPromise.then(result => Preferences.onDOMContentLoaded(result));
+  window.addEventListener("toggle", Preferences);
   window.addEventListener("change", Preferences);
   window.addEventListener("command", Preferences);
   window.addEventListener("dialogaccept", Preferences);
@@ -275,7 +333,7 @@ const Preferences = window.Preferences = (function() {
   window.addEventListener("unload", Preferences, { once: true });
 
   class Preference extends EventEmitter {
-    constructor({ id, name, type, inverted, disabled }) {
+    constructor({ id, type, inverted }) {
       super();
       this.on("change", this.onChange.bind(this));
 
@@ -285,32 +343,28 @@ const Preferences = window.Preferences = (function() {
       this.batching = false;
 
       this.id = id;
-      this._name = name || this.id;
       this.type = type;
       this.inverted = !!inverted;
-      this._disabled = !!disabled;
-
-      // if the element has been inserted without the name attribute set,
-      // we have nothing to do here
-      if (!this.name) {
-        throw new Error(`preference with id '${id}' doesn't have name`);
-      }
 
       // In non-instant apply mode, we must try and use the last saved state
       // from any previous opens of a child dialog instead of the value from
       // preferences, to pick up any edits a user may have made.
 
-      if (Preferences.type == "child" && window.opener &&
-          window.opener.Preferences &&
-          Services.scriptSecurityManager.isSystemPrincipal(window.opener.document.nodePrincipal)) {
+      if (
+        Preferences.type == "child" &&
+        window.opener &&
+        window.opener.Preferences &&
+        window.opener.document.nodePrincipal.isSystemPrincipal
+      ) {
         // Try to find the preference in the parent window.
-        const preference = window.opener.Preferences.get(this.name);
+        const preference = window.opener.Preferences.get(this.id);
 
         // Don't use the value setter here, we don't want updateElements to be
         // prematurely fired.
         this._value = preference ? preference.value : this.valueFromPreferences;
-      } else
+      } else {
         this._value = this.valueFromPreferences;
+      }
     }
 
     reset() {
@@ -319,36 +373,37 @@ const Preferences = window.Preferences = (function() {
     }
 
     _reportUnknownType() {
-      const msg = `Preference with id=${this.id} and name=${this.name} has unknown type ${this.type}.`;
+      const msg = `Preference with id=${this.id} has unknown type ${this.type}.`;
       Services.console.logStringMessage(msg);
     }
 
     setElementValue(aElement) {
-      if (this.locked)
+      if (this.locked) {
         aElement.disabled = true;
-
-      if (!this.isElementEditable(aElement))
-        return;
-
-      let rv = undefined;
-      if (aElement.hasAttribute("onsyncfrompreference")) {
-        // Value changed, synthesize an event
-        try {
-          const event = document.createEvent("Events");
-          event.initEvent("syncfrompreference", true, true);
-          const f = new Function("event",
-                               aElement.getAttribute("onsyncfrompreference"));
-          rv = f.call(aElement, event);
-        } catch (e) {
-          Cu.reportError(e);
+      }
+      if (aElement.labels?.length) {
+        for (let label of aElement.labels) {
+          label.toggleAttribute("disabled", this.locked);
         }
       }
+
+      if (!this.isElementEditable(aElement)) {
+        return;
+      }
+
+      let rv = undefined;
+
+      if (Preferences._syncFromPrefListeners.has(aElement)) {
+        rv = Preferences._syncFromPrefListeners.get(aElement)(aElement);
+      }
       let val = rv;
-      if (val === undefined)
+      if (val === undefined) {
         val = Preferences.instantApply ? this.valueFromPreferences : this.value;
+      }
       // if the preference is marked for reset, show default value in UI
-      if (val === undefined)
+      if (val === undefined) {
         val = this.defaultValue;
+      }
 
       /**
        * Initialize a UI element property with a value. Handles the case
@@ -360,13 +415,14 @@ const Preferences = window.Preferences = (function() {
       function setValue(element, attribute, value) {
         if (attribute in element) {
           element[attribute] = value;
-        } else if (attribute === "checked") {
+        } else if (attribute === "checked" || attribute === "pressed") {
           // The "checked" attribute can't simply be set to the specified value;
           // it has to be set if the value is true and removed if the value
           // is false in order to be interpreted correctly by the element.
           if (value) {
-            // We can set it to anything; convention is to set it to itself.
-            element.setAttribute(attribute, attribute);
+            // In theory we can set it to anything; however xbl implementation
+            // of `checkbox` only works with "true".
+            element.setAttribute(attribute, "true");
           } else {
             element.removeAttribute(attribute);
           }
@@ -374,34 +430,27 @@ const Preferences = window.Preferences = (function() {
           element.setAttribute(attribute, value);
         }
       }
-      if (aElement.localName == "checkbox" ||
-          aElement.localName == "listitem")
+      if (
+        aElement.localName == "checkbox" ||
+        (aElement.localName == "input" && aElement.type == "checkbox")
+      ) {
         setValue(aElement, "checked", val);
-      else if (aElement.localName == "colorpicker")
-        setValue(aElement, "color", val);
-      else if (aElement.localName == "textbox") {
-        // XXXmano Bug 303998: Avoid a caret placement issue if either the
-        // preference observer or its setter calls updateElements as a result
-        // of the input event handler.
-        if (aElement.value !== val)
-          setValue(aElement, "value", val);
-      } else
+      } else if (aElement.localName == "moz-toggle") {
+        setValue(aElement, "pressed", val);
+      } else {
         setValue(aElement, "value", val);
+      }
     }
 
     getElementValue(aElement) {
-      if (aElement.hasAttribute("onsynctopreference")) {
-        // Value changed, synthesize an event
+      if (Preferences._syncToPrefListeners.has(aElement)) {
         try {
-          const event = document.createEvent("Events");
-          event.initEvent("synctopreference", true, true);
-          const f = new Function("event",
-                               aElement.getAttribute("onsynctopreference"));
-          const rv = f.call(aElement, event);
-          if (rv !== undefined)
+          const rv = Preferences._syncToPrefListeners.get(aElement)(aElement);
+          if (rv !== undefined) {
             return rv;
+          }
         } catch (e) {
-          Cu.reportError(e);
+          console.error(e);
         }
       }
 
@@ -412,70 +461,68 @@ const Preferences = window.Preferences = (function() {
        * an attribute, as is the case before a binding has been attached.
        */
       function getValue(element, attribute) {
-        if (attribute in element)
+        if (attribute in element) {
           return element[attribute];
+        }
         return element.getAttribute(attribute);
       }
       let value;
-      if (aElement.localName == "checkbox" ||
-          aElement.localName == "listitem")
+      if (
+        aElement.localName == "checkbox" ||
+        (aElement.localName == "input" && aElement.type == "checkbox")
+      ) {
         value = getValue(aElement, "checked");
-      else if (aElement.localName == "colorpicker")
-        value = getValue(aElement, "color");
-      else
+      } else if (aElement.localName == "moz-toggle") {
+        value = getValue(aElement, "pressed");
+      } else {
         value = getValue(aElement, "value");
+      }
 
       switch (this.type) {
-      case "int":
-        return parseInt(value, 10) || 0;
-      case "bool":
-        return typeof(value) == "boolean" ? value : value == "true";
+        case "int":
+          return parseInt(value, 10) || 0;
+        case "bool":
+          return typeof value == "boolean" ? value : value == "true";
       }
       return value;
     }
 
     isElementEditable(aElement) {
       switch (aElement.localName) {
-      case "checkbox":
-      case "colorpicker":
-      case "radiogroup":
-      case "textbox":
-      case "listitem":
-      case "listbox":
-      case "menulist":
-        return true;
+        case "checkbox":
+        case "input":
+        case "radiogroup":
+        case "textarea":
+        case "menulist":
+        case "moz-toggle":
+          return true;
       }
-      return aElement.getAttribute("preference-editable") == "true";
+      return false;
     }
 
     updateElements() {
-      if (!this.id)
+      let startTime = performance.now();
+
+      if (!this.id) {
         return;
+      }
 
-      // This "change" event handler tracks changes made to preferences by
-      // sources other than the user in this window.
       const elements = getElementsByAttribute("preference", this.id);
-      for (const element of elements)
+      for (const element of elements) {
         this.setElementValue(element);
+      }
+
+      ChromeUtils.addProfilerMarker(
+        "Preferences",
+        { startTime, captureStack: true },
+        `updateElements for ${this.id}`
+      );
     }
 
     onChange() {
       this.updateElements();
     }
 
-    get name() {
-      return this._name;
-    }
-
-    set name(val) {
-      if (val == this.name)
-        return val;
-
-      this._name = val;
-
-      return val;
-    }
-
     get value() {
       return this._value;
     }
@@ -483,42 +530,39 @@ const Preferences = window.Preferences = (function() {
     set value(val) {
       if (this.value !== val) {
         this._value = val;
-        if (Preferences.instantApply)
+        if (Preferences.instantApply) {
           this.valueFromPreferences = val;
+        }
         this.emit("change");
       }
-      return val;
     }
 
     get locked() {
-      return Services.prefs.prefIsLocked(this.name);
+      return Services.prefs.prefIsLocked(this.id);
     }
 
-    get disabled() {
-      return this._disabled;
-    }
+    updateControlDisabledState(val) {
+      if (!this.id) {
+        return;
+      }
 
-    set disabled(val) {
-      this._disabled = !!val;
-
-      if (!this.id)
-        return val;
+      val = val || this.locked;
 
       const elements = getElementsByAttribute("preference", this.id);
       for (const element of elements) {
         element.disabled = val;
 
         const labels = getElementsByAttribute("control", element.id);
-        for (const label of labels)
+        for (const label of labels) {
           label.disabled = val;
+        }
       }
-
-      return val;
     }
 
     get hasUserValue() {
-      return Services.prefs.prefHasUserValue(this.name) &&
-             this.value !== undefined;
+      return (
+        Services.prefs.prefHasUserValue(this.id) && this.value !== undefined
+      );
     }
 
     get defaultValue() {
@@ -536,91 +580,97 @@ const Preferences = window.Preferences = (function() {
       try {
         // Force a resync of value with preferences.
         switch (this.type) {
-        case "int":
-          return this._branch.getIntPref(this.name);
-        case "bool": {
-          const val = this._branch.getBoolPref(this.name);
-          return this.inverted ? !val : val;
+          case "int":
+            return this._branch.getIntPref(this.id);
+          case "bool": {
+            const val = this._branch.getBoolPref(this.id);
+            return this.inverted ? !val : val;
+          }
+          case "wstring":
+            return this._branch.getComplexValue(
+              this.id,
+              Ci.nsIPrefLocalizedString
+            ).data;
+          case "string":
+          case "unichar":
+            return this._branch.getStringPref(this.id);
+          case "fontname": {
+            const family = this._branch.getStringPref(this.id);
+            const fontEnumerator = Cc[
+              "@mozilla.org/gfx/fontenumerator;1"
+            ].createInstance(Ci.nsIFontEnumerator);
+            return fontEnumerator.getStandardFamilyName(family);
+          }
+          case "file": {
+            const f = this._branch.getComplexValue(this.id, Ci.nsIFile);
+            return f;
+          }
+          default:
+            this._reportUnknownType();
         }
-        case "wstring":
-          return this._branch
-                     .getComplexValue(this.name, Ci.nsIPrefLocalizedString)
-                     .data;
-        case "string":
-        case "unichar":
-          return this._branch.getStringPref(this.name);
-        case "fontname": {
-          const family = this._branch.getStringPref(this.name);
-          const fontEnumerator = Cc["@mozilla.org/gfx/fontenumerator;1"]
-                               .createInstance(Ci.nsIFontEnumerator);
-          return fontEnumerator.getStandardFamilyName(family);
-        }
-        case "file": {
-          const f = this._branch
-                      .getComplexValue(this.name, Ci.nsIFile);
-          return f;
-        }
-        default:
-          this._reportUnknownType();
-        }
-      } catch (e) { }
+      } catch (e) {}
       return null;
     }
 
     set valueFromPreferences(val) {
       // Exit early if nothing to do.
-      if (this.readonly || this.valueFromPreferences == val)
-        return val;
+      if (this.readonly || this.valueFromPreferences == val) {
+        return;
+      }
 
       // The special value undefined means 'reset preference to default'.
       if (val === undefined) {
-        Services.prefs.clearUserPref(this.name);
-        return val;
+        Services.prefs.clearUserPref(this.id);
+        return;
       }
 
       // Force a resync of preferences with value.
       switch (this.type) {
-      case "int":
-        Services.prefs.setIntPref(this.name, val);
-        break;
-      case "bool":
-        Services.prefs.setBoolPref(this.name, this.inverted ? !val : val);
-        break;
-      case "wstring": {
-        const pls = Cc["@mozilla.org/pref-localizedstring;1"]
-                  .createInstance(Ci.nsIPrefLocalizedString);
-        pls.data = val;
-        Services.prefs
-            .setComplexValue(this.name, Ci.nsIPrefLocalizedString, pls);
-        break;
-      }
-      case "string":
-      case "unichar":
-      case "fontname":
-        Services.prefs.setStringPref(this.name, val);
-        break;
-      case "file": {
-        let lf;
-        if (typeof(val) == "string") {
-          lf = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsIFile);
-          lf.persistentDescriptor = val;
-          if (!lf.exists())
-            lf.initWithPath(val);
-        } else
-          lf = val.QueryInterface(Ci.nsIFile);
-        Services.prefs
-            .setComplexValue(this.name, Ci.nsIFile, lf);
-        break;
-      }
-      default:
-        this._reportUnknownType();
+        case "int":
+          Services.prefs.setIntPref(this.id, val);
+          break;
+        case "bool":
+          Services.prefs.setBoolPref(this.id, this.inverted ? !val : val);
+          break;
+        case "wstring": {
+          const pls = Cc["@mozilla.org/pref-localizedstring;1"].createInstance(
+            Ci.nsIPrefLocalizedString
+          );
+          pls.data = val;
+          Services.prefs.setComplexValue(
+            this.id,
+            Ci.nsIPrefLocalizedString,
+            pls
+          );
+          break;
+        }
+        case "string":
+        case "unichar":
+        case "fontname":
+          Services.prefs.setStringPref(this.id, val);
+          break;
+        case "file": {
+          let lf;
+          if (typeof val == "string") {
+            lf = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsIFile);
+            lf.persistentDescriptor = val;
+            if (!lf.exists()) {
+              lf.initWithPath(val);
+            }
+          } else {
+            lf = val.QueryInterface(Ci.nsIFile);
+          }
+          Services.prefs.setComplexValue(this.id, Ci.nsIFile, lf);
+          break;
+        }
+        default:
+          this._reportUnknownType();
       }
       if (!this.batching) {
         Services.prefs.savePrefFile(null);
       }
-      return val;
     }
   }
 
   return Preferences;
-}());
+})());
