@@ -32,6 +32,7 @@ var gPopupTests = null;
 var gTestIndex = -1;
 var gTestStepIndex = 0;
 var gTestEventIndex = 0;
+var gActualEvents = [];
 var gAutoHide = false;
 var gExpectedEventDetails = null;
 var gExpectedTriggerNode = null;
@@ -51,6 +52,9 @@ function startPopupTests(tests) {
   document.addEventListener("DOMMenuBarActive", eventOccurred);
   document.addEventListener("DOMMenuBarInactive", eventOccurred);
 
+  // This is useful to explicitly finish a test that shouldn't trigger events.
+  document.addEventListener("TestDone", eventOccurred);
+
   gPopupTests = tests;
   gWindowUtils = SpecialPowers.getDOMWindowUtils(window);
 
@@ -78,6 +82,14 @@ function ok(condition, message) {
   }
 }
 
+function info(message) {
+  if (window.opener) {
+    window.opener.SimpleTest.info(message);
+  } else {
+    SimpleTest.info(message);
+  }
+}
+
 function is(left, right, message) {
   if (window.opener) {
     window.opener.SimpleTest.is(left, right, message);
@@ -123,6 +135,8 @@ function eventOccurred(event) {
       return;
     }
 
+    gActualEvents.push(`${event.type} ${event.target.id}`);
+
     var eventitem = events[gTestEventIndex].split(" ");
     var matches;
     if (eventitem[1] == "#tooltip") {
@@ -190,17 +204,6 @@ function eventOccurred(event) {
         gExpectedTriggerNode,
         test.testname + " popupshowing triggerNode"
       );
-      var isTooltip = event.target.localName == "tooltip";
-      is(
-        document.popupNode,
-        isTooltip ? null : gExpectedTriggerNode,
-        test.testname + " popupshowing document.popupNode"
-      );
-      is(
-        document.tooltipNode,
-        isTooltip ? gExpectedTriggerNode : null,
-        test.testname + " popupshowing document.tooltipNode"
-      );
     }
 
     if (expectedState) {
@@ -216,6 +219,8 @@ function eventOccurred(event) {
       if (events.length <= gTestEventIndex) {
         setTimeout(checkResult, 0);
       }
+    } else {
+      info(`Actual events so far: ${JSON.stringify(gActualEvents)}`);
     }
   }
 }
@@ -241,7 +246,9 @@ async function checkResult() {
 }
 
 function goNextStep() {
+  info(`events: ${JSON.stringify(gActualEvents)}`);
   gTestEventIndex = 0;
+  gActualEvents = [];
 
   var step = null;
   var test = gPopupTests[gTestIndex];
@@ -260,7 +267,7 @@ function goNextStep() {
 function goNext() {
   // We want to continue after the next animation frame so that
   // we're in a stable state and don't get spurious mouse events at unexpected targets.
-  window.requestAnimationFrame(function() {
+  window.requestAnimationFrame(function () {
     setTimeout(goNextStepSync, 0);
   });
 }
@@ -281,6 +288,7 @@ function goNextStepSync() {
     var test = gPopupTests[gTestIndex];
     // Set the location hash so it's easy to see which test is running
     document.location.hash = test.testname;
+    info("Starting " + test.testname);
 
     // skip the test if the condition returns false
     if ("condition" in test && !test.condition()) {
@@ -402,8 +410,8 @@ function convertPosition(anchor, align) {
  * which can shift the bottom/right up to 0.5px from its "ideal" location,
  * and could cause it to round differently. (See bug 622507.)
  */
-function isWithinHalfPixel(a, b) {
-  return Math.abs(a - b) <= 0.5;
+function isWithinHalfPixel(a, b, message) {
+  ok(Math.abs(a - b) <= 0.5, `${message}: ${a}, ${b}`);
 }
 
 function compareEdge(anchor, popup, edge, offsetX, offsetY, testname) {
@@ -413,8 +421,6 @@ function compareEdge(anchor, popup, edge, offsetX, offsetY, testname) {
 
   var anchorrect = anchor.getBoundingClientRect();
   var popuprect = popup.getBoundingClientRect();
-  var check1 = false,
-    check2 = false;
 
   if (gPopupWidth == -1) {
     ok(
@@ -471,14 +477,30 @@ function compareEdge(anchor, popup, edge, offsetX, offsetY, testname) {
         cornerX += offsetX;
         cornerY += offsetY;
         break;
+      case "topcenter":
+        cornerX += -popuprect.width / 2 + offsetX;
+        cornerY += offsetY;
+        break;
       case "topright":
         cornerX += -popuprect.width + offsetX;
         cornerY += offsetY;
         break;
+      case "leftcenter":
+        cornerX += offsetX;
+        cornerY += -popuprect.height / 2 + offsetY;
+        break;
+      case "rightcenter":
+        cornerX += -popuprect.width + offsetX;
+        cornerY += -popuprect.height / 2 + offsetY;
+        break;
       case "bottomleft":
         cornerX += offsetX;
         cornerY += -popuprect.height + offsetY;
         break;
+      case "bottomcenter":
+        cornerX += -popuprect.width / 2 + offsetX;
+        cornerY += -popuprect.height + offsetY;
+        break;
       case "bottomright":
         cornerX += -popuprect.width + offsetX;
         cornerY += -popuprect.height + offsetY;
@@ -498,51 +520,69 @@ function compareEdge(anchor, popup, edge, offsetX, offsetY, testname) {
     return;
   }
 
-  if (edge == "after_pointer") {
-    is(
-      Math.round(popuprect.left),
-      Math.round(anchorrect.left) + offsetX,
-      testname + " x position"
-    );
-    is(
-      Math.round(popuprect.top),
-      Math.round(anchorrect.top) + offsetY + 21,
-      testname + " y position"
-    );
-    return;
-  }
-
   if (edge == "overlap") {
-    ok(
-      Math.round(anchorrect.left) + offsetY == Math.round(popuprect.left) &&
-        Math.round(anchorrect.top) + offsetY == Math.round(popuprect.top),
-      testname + " position"
+    is(
+      Math.round(anchorrect.left) + offsetY,
+      Math.round(popuprect.left),
+      testname + " position1"
+    );
+    is(
+      Math.round(anchorrect.top) + offsetY,
+      Math.round(popuprect.top),
+      testname + " position2"
     );
     return;
   }
 
   if (edge.indexOf("before") == 0) {
-    check1 = isWithinHalfPixel(anchorrect.top + offsetY, popuprect.bottom);
+    isWithinHalfPixel(
+      anchorrect.top + offsetY,
+      popuprect.bottom,
+      testname + " position1"
+    );
   } else if (edge.indexOf("after") == 0) {
-    check1 =
-      Math.round(anchorrect.bottom) + offsetY == Math.round(popuprect.top);
+    is(
+      Math.round(anchorrect.bottom) + offsetY,
+      Math.round(popuprect.top),
+      testname + " position1"
+    );
   } else if (edge.indexOf("start") == 0) {
-    check1 = isWithinHalfPixel(anchorrect.left + offsetX, popuprect.right);
+    isWithinHalfPixel(
+      anchorrect.left + offsetX,
+      popuprect.right,
+      testname + " position1"
+    );
   } else if (edge.indexOf("end") == 0) {
-    check1 =
-      Math.round(anchorrect.right) + offsetX == Math.round(popuprect.left);
+    is(
+      Math.round(anchorrect.right) + offsetX,
+      Math.round(popuprect.left),
+      testname + " position1"
+    );
   }
 
   if (0 < edge.indexOf("before")) {
-    check2 = Math.round(anchorrect.top) + offsetY == Math.round(popuprect.top);
+    is(
+      Math.round(anchorrect.top) + offsetY,
+      Math.round(popuprect.top),
+      testname + " position2"
+    );
   } else if (0 < edge.indexOf("after")) {
-    check2 = isWithinHalfPixel(anchorrect.bottom + offsetY, popuprect.bottom);
+    isWithinHalfPixel(
+      anchorrect.bottom + offsetY,
+      popuprect.bottom,
+      testname + " position2"
+    );
   } else if (0 < edge.indexOf("start")) {
-    check2 =
-      Math.round(anchorrect.left) + offsetX == Math.round(popuprect.left);
+    is(
+      Math.round(anchorrect.left) + offsetX,
+      Math.round(popuprect.left),
+      testname + " position2"
+    );
   } else if (0 < edge.indexOf("end")) {
-    check2 = isWithinHalfPixel(anchorrect.right + offsetX, popuprect.right);
+    isWithinHalfPixel(
+      anchorrect.right + offsetX,
+      popuprect.right,
+      testname + " position2"
+    );
   }
-
-  ok(check1 && check2, testname + " position");
 }
