@@ -26,10 +26,17 @@
     RemoteWebNavigation: "resource://gre/modules/RemoteWebNavigation.sys.mjs",
   });
 
-  XPCOMUtils.defineLazyGetter(lazy, "blankURI", () =>
+  ChromeUtils.defineLazyGetter(lazy, "blankURI", () =>
     Services.io.newURI("about:blank")
   );
 
+  XPCOMUtils.defineLazyServiceGetter(
+    lazy,
+    "contentAnalysis",
+    "@mozilla.org/contentanalysis;1",
+    Ci.nsIContentAnalysis
+  );
+
   let lazyPrefs = {};
   XPCOMUtils.defineLazyPreferenceGetter(
     lazyPrefs,
@@ -43,9 +50,9 @@
       // available, in which case we return null. We replace this getter
       // when the module becomes available (should be on delayed startup
       // when the first browser window loads, via BrowserGlue.sys.mjs).
-      const kURL = "resource:///modules/ProcessHangMonitor.jsm";
-      if (Cu.isModuleLoaded(kURL)) {
-        let { ProcessHangMonitor } = ChromeUtils.import(kURL);
+      const kURL = "resource:///modules/ProcessHangMonitor.sys.mjs";
+      if (Cu.isESModuleLoaded(kURL)) {
+        let { ProcessHangMonitor } = ChromeUtils.importESModule(kURL);
         // eslint-disable-next-line mozilla/valid-lazy
         Object.defineProperty(lazy, "ProcessHangMonitor", {
           value: ProcessHangMonitor,
@@ -118,7 +125,7 @@
       this.mIconURL = null;
       this.lastURI = null;
 
-      XPCOMUtils.defineLazyGetter(this, "popupBlocker", () => {
+      ChromeUtils.defineLazyGetter(this, "popupBlocker", () => {
         return new lazy.PopupBlocker(this);
       });
 
@@ -159,6 +166,131 @@
       this.addEventListener(
         "drop",
         event => {
+          if (lazy.contentAnalysis.isActive) {
+            let dragService = Cc[
+              "@mozilla.org/widget/dragservice;1"
+            ].getService(Ci.nsIDragService);
+            let dragSession = dragService.getCurrentSession(window);
+            if (!dragSession) {
+              return;
+            }
+
+            // Don't check items from drags that take place inside of a single
+            // frame, or in a same-origin iframe hierarchy.  Drag sessions with an
+            // external source have no sourceWindowContext and must be checked.
+            let sourceWC = dragSession.sourceWindowContext;
+            let targetWC = this.browsingContext?.currentWindowContext;
+            if (!targetWC) {
+              return;
+            }
+            if (
+              sourceWC &&
+              sourceWC.browsingContext.top == targetWC.browsingContext.top &&
+              targetWC.documentPrincipal?.subsumes(sourceWC.documentPrincipal)
+            ) {
+              return;
+            }
+
+            // Create a request for each text and file item in the DataTransfer
+            try {
+              // Tell browser to record the event target and to delay EndDragSession
+              // until the content analysis results are given.
+              dragSession.sendStoreDropTargetAndDelayEndDragSession(event);
+
+              // Submit a content analysis request for each checkable entry in the
+              // DataTransfer and stop dispatching this drop event.  Reissue the
+              // drop if all requests are permitted.
+              let caPromises = [];
+              let items = event.dataTransfer.items;
+              for (let elt of items) {
+                const kTextMimeTypes = [
+                  "text/plain",
+                  "text/html",
+                  "application/x-moz-nativehtml",
+                ];
+
+                let requestFields;
+                if (
+                  elt.kind === "string" &&
+                  kTextMimeTypes.includes(elt.type)
+                ) {
+                  let str = event.dataTransfer.getData(elt.type);
+                  if (!str) {
+                    continue;
+                  }
+                  requestFields = {
+                    analysisType: Ci.nsIContentAnalysisRequest.eBulkDataEntry,
+                    operationTypeForDisplay:
+                      Ci.nsIContentAnalysisRequest.eDroppedText,
+                    textContent: str,
+                  };
+                } else if (elt.kind === "file") {
+                  let file = elt.getAsFile();
+                  requestFields = {
+                    analysisType: Ci.nsIContentAnalysisRequest.eFileAttached,
+                    operationTypeForDisplay:
+                      Ci.nsIContentAnalysisRequest.eCustomDisplayString,
+                    operationDisplayString: file.name,
+                    filePath: file.mozFullPath,
+                  };
+                } else {
+                  // Unrecognized data type -- don't send to content analysis
+                  continue;
+                }
+
+                caPromises.push(
+                  lazy.contentAnalysis.analyzeContentRequest(
+                    {
+                      requestToken: Services.uuid.generateUUID().toString(),
+                      resources: [],
+                      url: lazy.contentAnalysis.getURIForDropEvent(event),
+                      windowGlobalParent:
+                        this.browsingContext.currentWindowContext,
+                      ...requestFields,
+                    },
+                    true /* autoAcknowledge */
+                  )
+                );
+              }
+
+              if (!caPromises.length) {
+                // Nothing was analyzable.
+                dragSession.sendDispatchToDropTargetAndResumeEndDragSession(
+                  true
+                );
+                return;
+              }
+
+              // Only permit the drop if all requests were approved.  Issue dragexit
+              // instead of drop if CA rejected the content or there was an error.
+              Promise.all(caPromises).then(
+                caResults => {
+                  let allApproved = caResults.reduce((prev, current) => {
+                    return prev && current.shouldAllowContent;
+                  }, true);
+                  dragSession.sendDispatchToDropTargetAndResumeEndDragSession(
+                    allApproved
+                  );
+                },
+                () => {
+                  dragSession.sendDispatchToDropTargetAndResumeEndDragSession(
+                    false
+                  );
+                }
+              );
+
+              // Do not allow this drop to continue dispatch.
+              event.preventDefault();
+              event.stopPropagation();
+            } catch {
+              // On internal error, deny any drop.  CA has its own behavior to
+              // handle internal errors, like a lost connection to the agent, but
+              // we are more strict when facing errors here.
+              event.preventDefault();
+              event.stopPropagation();
+            }
+          }
+
           // No need to handle "drop" in e10s, since nsDocShellTreeOwner.cpp in the child process
           // handles that case using "@mozilla.org/content/dropped-link-handler;1" service.
           if (
@@ -277,8 +409,6 @@
 
       this.mPrefs = Services.prefs;
 
-      this._mStrBundle = null;
-
       this._audioMuted = false;
 
       this._hasAnyPlayingMediaBeenBlocked = false;
@@ -687,17 +817,6 @@
       return !!this.browsingContext.opener;
     }
 
-    get mStrBundle() {
-      if (!this._mStrBundle) {
-        // need to create string bundle manually instead of using <xul:stringbundle/>
-        // see bug 63370 for details
-        this._mStrBundle = Services.strings.createBundle(
-          "chrome://global/locale/browser.properties"
-        );
-      }
-      return this._mStrBundle;
-    }
-
     get audioMuted() {
       return this._audioMuted;
     }
@@ -885,7 +1004,7 @@
       this.webProgress.removeProgressListener(aListener);
     }
 
-    onPageHide(aEvent) {
+    onPageHide() {
       // If we're browsing from the tab crashed UI to a URI that keeps
       // this browser non-remote, we'll handle that here.
       lazy.SessionStore?.maybeExitCrashedState(this);
@@ -1228,7 +1347,7 @@
       }
     }
 
-    createAboutBlankContentViewer(aPrincipal, aPartitionedPrincipal) {
+    createAboutBlankDocumentViewer(aPrincipal, aPartitionedPrincipal) {
       let principal = lazy.BrowserUtils.principalWithMatchingOA(
         aPrincipal,
         this.contentPrincipal
@@ -1239,12 +1358,12 @@
       );
 
       if (this.isRemoteBrowser) {
-        this.frameLoader.remoteTab.createAboutBlankContentViewer(
+        this.frameLoader.remoteTab.createAboutBlankDocumentViewer(
           principal,
           partitionedPrincipal
         );
       } else {
-        this.docShell.createAboutBlankContentViewer(
+        this.docShell.createAboutBlankDocumentViewer(
           principal,
           partitionedPrincipal
         );
@@ -1610,6 +1729,7 @@
             "_contentPartitionedPrincipal",
             "_isSyntheticDocument",
             "_originalURI",
+            "_userTypedValue",
           ]
         );
       }
@@ -1676,11 +1796,11 @@
         return;
       }
 
-      if (!this.docShell || !this.docShell.contentViewer) {
+      if (!this.docShell || !this.docShell.docViewer) {
         aCallback(false);
         return;
       }
-      aCallback(this.docShell.contentViewer.inPermitUnload);
+      aCallback(this.docShell.docViewer.inPermitUnload);
     }
 
     async asyncPermitUnload(action) {
@@ -1752,11 +1872,11 @@
         throw result;
       }
 
-      if (!this.docShell || !this.docShell.contentViewer) {
+      if (!this.docShell || !this.docShell.docViewer) {
         return { permitUnload: true };
       }
       return {
-        permitUnload: this.docShell.contentViewer.permitUnload(),
+        permitUnload: this.docShell.docViewer.permitUnload(),
       };
     }
 
@@ -1931,7 +2051,7 @@
     // Called immediately after changing remoteness.  If this method returns
     // `true`, Gecko will assume frontend handled resuming the load, and will
     // not attempt to resume the load itself.
-    afterChangeRemoteness(browser, redirectLoadSwitchId) {
+    afterChangeRemoteness() {
       /* no-op unless replaced */
       return false;
     }
