@@ -9,6 +9,10 @@
 // This is loaded into all XUL windows. Wrap in a block to prevent
 // leaking to window scope.
 {
+  const { AppConstants } = ChromeUtils.importESModule(
+    "resource://gre/modules/AppConstants.sys.mjs"
+  );
+
   class MozTreeChildren extends MozElements.BaseControl {
     constructor() {
       super();
@@ -194,36 +198,19 @@
   customElements.define("treechildren", MozTreeChildren);
 
   class MozTreecolPicker extends MozElements.BaseControl {
+    static get markup() {
+      return `
+      <button class="tree-columnpicker-button"/>
+      <menupopup anonid="popup">
+        <menuseparator anonid="menuseparator"/>
+        <menuitem anonid="menuitem" data-l10n-id="tree-columnpicker-restore-order"/>
+      </menupopup>
+      `;
+    }
     constructor() {
       super();
 
-      this.addEventListener("command", event => {
-        if (event.originalTarget == this) {
-          var popup = this.querySelector('[anonid="popup"]');
-          this.buildPopup(popup);
-          popup.openPopup(this, "after_end");
-        } else {
-          var tree = this.parentNode.parentNode;
-          tree.stopEditing(true);
-          var menuitem = this.querySelector('[anonid="menuitem"]');
-          if (event.originalTarget == menuitem) {
-            tree.columns.restoreNaturalOrder();
-            this.removeAttribute("ordinal");
-            tree._ensureColumnOrder();
-          } else {
-            var colindex = event.originalTarget.getAttribute("colindex");
-            var column = tree.columns[colindex];
-            if (column) {
-              var element = column.element;
-              if (element.getAttribute("hidden") == "true") {
-                element.setAttribute("hidden", "false");
-              } else {
-                element.setAttribute("hidden", "true");
-              }
-            }
-          }
-        }
-      });
+      window.MozXULElement.insertFTLIfNeeded("toolkit/global/tree.ftl");
     }
 
     connectedCallback() {
@@ -232,18 +219,25 @@
       }
 
       this.textContent = "";
-      this.appendChild(
-        MozXULElement.parseXULToFragment(
-          `
-        <image class="tree-columnpicker-icon"></image>
-        <menupopup anonid="popup">
-          <menuseparator anonid="menuseparator"></menuseparator>
-          <menuitem anonid="menuitem" label="&restoreColumnOrder.label;"></menuitem>
-        </menupopup>
-      `,
-          ["chrome://global/locale/tree.dtd"]
-        )
-      );
+      this.appendChild(this.constructor.fragment);
+
+      let button = this.querySelector(".tree-columnpicker-button");
+      let popup = this.querySelector('[anonid="popup"]');
+      let menuitem = this.querySelector('[anonid="menuitem"]');
+
+      button.addEventListener("command", e => {
+        this.buildPopup(popup);
+        popup.openPopup(this, "after_end");
+        e.preventDefault();
+      });
+
+      menuitem.addEventListener("command", e => {
+        let tree = this.parentNode.parentNode;
+        tree.stopEditing(true);
+        this.style.order = "";
+        tree._ensureColumnOrder(tree.NATURAL_ORDER);
+        e.preventDefault();
+      });
     }
 
     buildPopup(aPopup) {
@@ -265,7 +259,7 @@
         // it is not being shown.
         var currElement = currCol.element;
         if (!currElement.hasAttribute("ignoreincolumnpicker")) {
-          var popupChild = document.createElement("menuitem");
+          var popupChild = document.createXULElement("menuitem");
           popupChild.setAttribute("type", "checkbox");
           var columnName =
             currElement.getAttribute("display") ||
@@ -284,6 +278,16 @@
               currElement.getAttribute("closemenu")
             );
           }
+
+          popupChild.addEventListener("command", function () {
+            let colindex = this.getAttribute("colindex");
+            let column = tree.columns[colindex];
+            if (column) {
+              var element = column.element;
+              element.hidden = !element.hidden;
+            }
+          });
+
           aPopup.insertBefore(popupChild, refChild);
         }
       }
@@ -298,6 +302,10 @@
   customElements.define("treecolpicker", MozTreecolPicker);
 
   class MozTreecol extends MozElements.BaseControl {
+    static get observedAttributes() {
+      return ["primary", ...super.observedAttributes];
+    }
+
     static get inheritedAttributes() {
       return {
         ".treecol-sortdirection": "sortdirection,hidden=hideheader",
@@ -305,11 +313,24 @@
       };
     }
 
-    get content() {
-      return MozXULElement.parseXULToFragment(`
-        <label class="treecol-text" flex="1" crop="right"></label>
+    static get markup() {
+      return `
+        <label class="treecol-text" flex="1" crop="end"></label>
         <image class="treecol-sortdirection"></image>
-    `);
+      `;
+    }
+
+    get _tree() {
+      return this.parentNode?.parentNode;
+    }
+
+    _invalidate() {
+      let tree = this._tree;
+      if (!tree || !XULTreeElement.isInstance(tree)) {
+        return;
+      }
+      tree.invalidate();
+      tree.columns?.invalidateColumns();
     }
 
     constructor() {
@@ -319,10 +340,10 @@
         if (event.button != 0) {
           return;
         }
-        if (this.parentNode.parentNode.enableColumnDrag) {
-          var xulns =
+        if (this._tree.enableColumnDrag) {
+          var XUL_NS =
             "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";
-          var cols = this.parentNode.getElementsByTagNameNS(xulns, "treecol");
+          var cols = this.parentNode.getElementsByTagNameNS(XUL_NS, "treecol");
 
           // only start column drag operation if there are at least 2 visible columns
           var visible = 0;
@@ -353,11 +374,11 @@
 
         // On Windows multiple clicking on tree columns only cycles one time
         // every 2 clicks.
-        if (/Win/.test(navigator.platform) && event.detail % 2 == 0) {
+        if (AppConstants.platform == "win" && event.detail % 2 == 0) {
           return;
         }
 
-        var tree = this.parentNode.parentNode;
+        var tree = this._tree;
         if (tree.columns) {
           tree.view.cycleHeader(tree.columns.getColumnFor(this));
         }
@@ -370,17 +391,38 @@
       }
 
       this.textContent = "";
-      this.appendChild(this.content);
+      this.appendChild(this.constructor.fragment);
       this.initializeAttributeInheritance();
+      if (this.hasAttribute("ordinal")) {
+        this.style.order = this.getAttribute("ordinal");
+      }
+      if (this.hasAttribute("width")) {
+        this.style.width = this.getAttribute("width") + "px";
+      }
+
+      this._resizeObserver = new ResizeObserver(() => {
+        this._invalidate();
+      });
+      this._resizeObserver.observe(this);
+    }
+
+    disconnectedCallback() {
+      this._resizeObserver?.unobserve(this);
+      this._resizeObserver = null;
+    }
+
+    attributeChangedCallback(name, oldValue, newValue) {
+      super.attributeChangedCallback(name, oldValue, newValue);
+      this._invalidate();
     }
 
     set ordinal(val) {
+      this.style.order = val;
       this.setAttribute("ordinal", val);
-      return val;
     }
 
     get ordinal() {
-      var val = this.getAttribute("ordinal");
+      var val = this.style.order;
       if (val == "") {
         return "1";
       }
@@ -473,7 +515,7 @@
       }
     }
 
-    _onDragMouseUp(aEvent) {
+    _onDragMouseUp() {
       var col = document.treecolDragging;
       if (!col) {
         return;
@@ -530,7 +572,7 @@
       window.removeEventListener("mouseup", col._onDragMouseUp, true);
       // we have to wait for the click event to fire before removing
       // cancelling handler
-      var clickHandler = function(handler) {
+      var clickHandler = function (handler) {
         window.removeEventListener("click", handler, true);
       };
       window.setTimeout(clickHandler, 0, col._onDragMouseClick);
@@ -552,6 +594,12 @@
       };
     }
 
+    static get markup() {
+      return `
+      <treecolpicker fixed="true"></treecolpicker>
+      `;
+    }
+
     connectedCallback() {
       if (this.delayConnectedCallback()) {
         return;
@@ -560,11 +608,7 @@
       this.setAttribute("slot", "treecols");
 
       if (!this.querySelector("treecolpicker")) {
-        this.appendChild(
-          MozXULElement.parseXULToFragment(`
-          <treecolpicker class="treecol-image" fixed="true"></treecolpicker>
-        `)
-        );
+        this.appendChild(this.constructor.fragment);
         this.initializeAttributeInheritance();
       }
 
@@ -583,38 +627,55 @@
   class MozTree extends MozElements.BaseControlMixin(
     MozElements.MozElementMixin(XULTreeElement)
   ) {
+    static get markup() {
+      return `
+      <html:link rel="stylesheet" href="chrome://global/content/widgets.css" />
+      <html:slot name="treecols"></html:slot>
+      <stack class="tree-stack" flex="1">
+        <hbox class="tree-rows" flex="1">
+          <hbox flex="1" class="tree-bodybox">
+            <html:slot name="treechildren"></html:slot>
+          </hbox>
+          <scrollbar height="0" minwidth="0" minheight="0" orient="vertical"
+                     class="hidevscroll-scrollbar scrollbar-topmost"
+                     ></scrollbar>
+        </hbox>
+        <html:input class="tree-input" type="text" hidden="true"/>
+      </stack>
+      <hbox class="hidehscroll-box">
+        <scrollbar orient="horizontal" flex="1" increment="16" class="scrollbar-topmost" ></scrollbar>
+        <scrollcorner class="hidevscroll-scrollcorner"></scrollcorner>
+      </hbox>
+      `;
+    }
+
     constructor() {
       super();
 
+      // These enumerated constants are used as the first argument to
+      // _ensureColumnOrder to specify what column ordering should be used.
+      this.CURRENT_ORDER = 0;
+      this.NATURAL_ORDER = 1; // The original order, which is the DOM ordering
+
       this.attachShadow({ mode: "open" });
-      this.shadowRoot.appendChild(
-        MozXULElement.parseXULToFragment(`
-        <html:link rel="stylesheet" href="chrome://global/content/widgets.css" />
-        <html:slot name="treecols"></html:slot>
-        <stack class="tree-stack" flex="1">
-          <hbox class="tree-rows" flex="1">
-            <hbox flex="1" class="tree-bodybox">
-              <html:slot name="treechildren"></html:slot>
-            </hbox>
-            <scrollbar height="0" minwidth="0" minheight="0" orient="vertical"
-                       class="hidevscroll-scrollbar"
-                       style="position:relative; z-index:2147483647;"
-                       oncontextmenu="event.stopPropagation(); event.preventDefault();"
-                       onclick="event.stopPropagation(); event.preventDefault();"
-                       ondblclick="event.stopPropagation();"
-                       oncommand="event.stopPropagation();"></scrollbar>
-          </hbox>
-          <textbox class="tree-input" left="0" top="0" hidden="true"></textbox>
-        </stack>
-        <hbox class="hidehscroll-box">
-          <scrollbar orient="horizontal" flex="1" increment="16" style="position:relative; z-index:2147483647;" oncontextmenu="event.stopPropagation(); event.preventDefault();" onclick="event.stopPropagation(); event.preventDefault();" ondblclick="event.stopPropagation();" oncommand="event.stopPropagation();"></scrollbar>
-          <scrollcorner class="hidevscroll-scrollcorner"
-                        oncontextmenu="event.stopPropagation(); event.preventDefault();"
-                        onclick="event.stopPropagation(); event.preventDefault();"
-                        ondblclick="event.stopPropagation();"
-                        oncommand="event.stopPropagation();"></scrollcorner>
-        </hbox>
-      `)
+      let handledElements = this.constructor.fragment.querySelectorAll(
+        "scrollbar,scrollcorner"
+      );
+      let stopAndPrevent = e => {
+        e.stopPropagation();
+        e.preventDefault();
+      };
+      let stopProp = e => e.stopPropagation();
+      for (let el of handledElements) {
+        el.addEventListener("click", stopAndPrevent);
+        el.addEventListener("contextmenu", stopAndPrevent);
+        el.addEventListener("dblclick", stopProp);
+        el.addEventListener("command", stopProp);
+      }
+      this.shadowRoot.appendChild(this.constructor.fragment);
+
+      this.#verticalScrollbar = this.shadowRoot.querySelector(
+        "scrollbar[orient='vertical']"
       );
     }
 
@@ -637,11 +698,10 @@
 
       this.setAttribute("hidevscroll", "true");
       this.setAttribute("hidehscroll", "true");
-      this.setAttribute("clickthrough", "never");
 
       this.initializeAttributeInheritance();
 
-      this.pageUpOrDownMovesSelection = !/Mac/.test(navigator.platform);
+      this.pageUpOrDownMovesSelection = AppConstants.platform != "macosx";
 
       this._inputField = null;
 
@@ -726,35 +786,26 @@
         }
       });
 
-      this.addEventListener("touchend", event => {
+      this.addEventListener("touchend", () => {
         this._touchY = -1;
       });
 
-      this.addEventListener("MozMousePixelScroll", event => {
-        if (
-          !(
-            this.getAttribute("allowunderflowscroll") == "true" &&
-            this.getAttribute("hidevscroll") == "true"
-          )
-        ) {
+      // This event doesn't retarget, so listen on the shadow DOM directly
+      this.shadowRoot.addEventListener("MozMousePixelScroll", event => {
+        if (this.#canScroll(event)) {
           event.preventDefault();
         }
       });
 
-      this.addEventListener("DOMMouseScroll", event => {
-        if (
-          !(
-            this.getAttribute("allowunderflowscroll") == "true" &&
-            this.getAttribute("hidevscroll") == "true"
-          )
-        ) {
-          event.preventDefault();
-        }
-
-        if (this._editingColumn) {
+      // This event doesn't retarget, so listen on the shadow DOM directly
+      this.shadowRoot.addEventListener("DOMMouseScroll", event => {
+        if (!this.#canScroll(event)) {
           return;
         }
-        if (event.axis == event.HORIZONTAL_AXIS) {
+
+        event.preventDefault();
+
+        if (this._editingColumn) {
           return;
         }
 
@@ -789,7 +840,7 @@
         }
       });
 
-      this.addEventListener("focus", event => {
+      this.addEventListener("focus", () => {
         this.focused = true;
         if (this.currentIndex == -1 && this.view.rowCount > 0) {
           this.currentIndex = this.getFirstVisibleRow();
@@ -800,7 +851,7 @@
         "blur",
         event => {
           this.focused = false;
-          if (event.originalTarget == this.inputField.inputField) {
+          if (event.target == this.inputField) {
             this.stopEditing(true);
           }
         },
@@ -812,6 +863,55 @@
           return;
         }
 
+        let toggleClose = () => {
+          if (this._editingColumn) {
+            return;
+          }
+
+          let row = this.currentIndex;
+          if (row < 0) {
+            return;
+          }
+
+          if (this.changeOpenState(this.currentIndex, false)) {
+            event.preventDefault();
+            return;
+          }
+
+          let parentIndex = this.view.getParentIndex(this.currentIndex);
+          if (parentIndex >= 0) {
+            this.view.selection.select(parentIndex);
+            this.ensureRowIsVisible(parentIndex);
+            event.preventDefault();
+          }
+        };
+
+        let toggleOpen = () => {
+          if (this._editingColumn) {
+            return;
+          }
+
+          let row = this.currentIndex;
+          if (row < 0) {
+            return;
+          }
+
+          if (this.changeOpenState(row, true)) {
+            event.preventDefault();
+            return;
+          }
+          let c = row + 1;
+          let view = this.view;
+          if (c < view.rowCount && view.getParentIndex(c) == row) {
+            // If already opened, select the first child.
+            // The getParentIndex test above ensures that the children
+            // are already populated and ready.
+            this.view.selection.timedSelect(c, this._selectDelay);
+            this.ensureRowIsVisible(c);
+            event.preventDefault();
+          }
+        };
+
         switch (event.keyCode) {
           case KeyEvent.DOM_VK_RETURN: {
             if (this._handleEnter(event)) {
@@ -830,50 +930,18 @@
             break;
           }
           case KeyEvent.DOM_VK_LEFT: {
-            if (this._editingColumn) {
-              return;
-            }
-
-            let row = this.currentIndex;
-            if (row < 0) {
-              return;
-            }
-
-            if (this.changeOpenState(this.currentIndex, false)) {
-              event.preventDefault();
-              return;
-            }
-            let parentIndex = this.view.getParentIndex(this.currentIndex);
-            if (parentIndex >= 0) {
-              this.view.selection.select(parentIndex);
-              this.ensureRowIsVisible(parentIndex);
-              event.preventDefault();
+            if (!this.isRTL) {
+              toggleClose();
+            } else {
+              toggleOpen();
             }
             break;
           }
           case KeyEvent.DOM_VK_RIGHT: {
-            if (this._editingColumn) {
-              return;
-            }
-
-            let row = this.currentIndex;
-            if (row < 0) {
-              return;
-            }
-
-            if (this.changeOpenState(row, true)) {
-              event.preventDefault();
-              return;
-            }
-            let c = row + 1;
-            let view = this.view;
-            if (c < view.rowCount && view.getParentIndex(c) == row) {
-              // If already opened, select the first child.
-              // The getParentIndex test above ensures that the children
-              // are already populated and ready.
-              this.view.selection.timedSelect(c, this._selectDelay);
-              this.ensureRowIsVisible(c);
-              event.preventDefault();
+            if (!this.isRTL) {
+              toggleOpen();
+            } else {
+              toggleClose();
             }
             break;
           }
@@ -987,13 +1055,16 @@
       return this.treeBody;
     }
 
+    get isRTL() {
+      return document.defaultView.getComputedStyle(this).direction == "rtl";
+    }
+
     set editable(val) {
       if (val) {
         this.setAttribute("editable", "true");
       } else {
         this.removeAttribute("editable");
       }
-      return val;
     }
 
     get editable() {
@@ -1004,22 +1075,23 @@
      */
     set selType(val) {
       this.setAttribute("seltype", val);
-      return val;
     }
 
     get selType() {
-      return this.getAttribute("seltype");
+      return this.getAttribute("seltype") || "";
     }
 
     set currentIndex(val) {
       if (this.view) {
-        return (this.view.selection.currentIndex = val);
+        this.view.selection.currentIndex = val;
       }
-      return val;
     }
 
     get currentIndex() {
-      return this.view ? this.view.selection.currentIndex : -1;
+      if (this.view && this.view.selection) {
+        return this.view.selection.currentIndex;
+      }
+      return -1;
     }
 
     set keepCurrentInView(val) {
@@ -1028,7 +1100,6 @@
       } else {
         this.removeAttribute("keepcurrentinview");
       }
-      return val;
     }
 
     get keepCurrentInView() {
@@ -1041,7 +1112,6 @@
       } else {
         this.removeAttribute("enableColumnDrag");
       }
-      return val;
     }
 
     get enableColumnDrag() {
@@ -1051,11 +1121,7 @@
     get inputField() {
       if (!this._inputField) {
         this._inputField = this.shadowRoot.querySelector(".tree-input");
-        this._inputField.addEventListener(
-          "blur",
-          () => this.stopEditing(true),
-          true
-        );
+        this._inputField.addEventListener("blur", () => this.stopEditing(true));
       }
       return this._inputField;
     }
@@ -1066,7 +1132,6 @@
       } else {
         this.removeAttribute("disableKeyNavigation");
       }
-      return val;
     }
 
     get disableKeyNavigation() {
@@ -1089,27 +1154,36 @@
       return this.getAttribute("_selectDelay") || 50;
     }
 
-    _ensureColumnOrder() {
+    // The first argument (order) can be either one of these constants:
+    //   this.CURRENT_ORDER
+    //   this.NATURAL_ORDER
+    _ensureColumnOrder(order = this.CURRENT_ORDER) {
       if (this.columns) {
         // update the ordinal position of each column to assure that it is
         // an odd number and 2 positions above its next sibling
         var cols = [];
 
-        for (
-          let col = this.columns.getFirstColumn();
-          col;
-          col = col.getNext()
-        ) {
-          cols.push(col.element);
+        if (order == this.CURRENT_ORDER) {
+          for (
+            let col = this.columns.getFirstColumn();
+            col;
+            col = col.getNext()
+          ) {
+            cols.push(col.element);
+          }
+        } else {
+          // order == this.NATURAL_ORDER
+          cols = this.getElementsByTagName("treecol");
         }
+
         for (let i = 0; i < cols.length; ++i) {
-          cols[i].setAttribute("ordinal", i * 2 + 1);
+          cols[i].ordinal = i * 2 + 1;
         }
         // update the ordinal positions of splitters to even numbers, so that
         // they are in between columns
         var splitters = this.getElementsByTagName("splitter");
         for (let i = 0; i < splitters.length; ++i) {
-          splitters[i].setAttribute("ordinal", (i + 1) * 2);
+          splitters[i].style.order = (i + 1) * 2;
         }
       }
     }
@@ -1152,12 +1226,14 @@
         for (i = 0; i < cols.length; ++i) {
           cols[i].ordinal = parseInt(cols[i].ordinal) - 2;
         }
+      } else {
+        return;
       }
+      this.columns.invalidateColumns();
     }
 
     _getColumnAtX(aX, aThresh, aPos) {
-      var isRTL =
-        document.defaultView.getComputedStyle(this).direction == "rtl";
+      let isRTL = this.isRTL;
 
       if (aPos) {
         aPos.value = isRTL ? "after" : "before";
@@ -1264,10 +1340,7 @@
       if (row < 0 || row >= this.view.rowCount || !column) {
         return false;
       }
-      if (
-        column.type != window.TreeColumn.TYPE_TEXT &&
-        column.type != window.TreeColumn.TYPE_PASSWORD
-      ) {
+      if (column.type !== window.TreeColumn.TYPE_TEXT) {
         return false;
       }
       if (column.cycler || !this.view.isEditable(row, column)) {
@@ -1292,32 +1365,34 @@
       // Calculate the top offset of the textbox.
       var style = window.getComputedStyle(input);
       var topadj = parseInt(style.borderTopWidth) + parseInt(style.paddingTop);
-      input.top = textRect.y - topadj;
+      input.style.top = `${textRect.y - topadj}px`;
 
       // The leftside of the textbox is aligned to the left side of the text
       // in LTR mode, and left side of the cell in RTL mode.
-      var left, widthdiff;
-      if (style.direction == "rtl") {
-        left = cellRect.x;
-        widthdiff = cellRect.x - textRect.x;
-      } else {
-        left = textRect.x;
-        widthdiff = textRect.x - cellRect.x;
-      }
+      let left = style.direction == "rtl" ? cellRect.x : textRect.x;
+      let scrollbarWidth = window.windowUtils.getBoundsWithoutFlushing(
+        this.#verticalScrollbar
+      ).width;
+      // Note: this won't be quite right in RTL for trees using twisties
+      // or indentation. bug 1708159 tracks fixing the implementation
+      // of getCoordsForCellItem which we called above so it provides
+      // better numbers in those cases.
+      let widthdiff = Math.abs(textRect.x - cellRect.x) - scrollbarWidth;
 
-      input.left = left;
-      input.height =
+      input.style.left = `${left}px`;
+      input.style.height = `${
         textRect.height +
         topadj +
         parseInt(style.borderBottomWidth) +
-        parseInt(style.paddingBottom);
-      input.width = cellRect.width - widthdiff;
+        parseInt(style.paddingBottom)
+      }px`;
+      input.style.width = `${cellRect.width - widthdiff}px`;
       input.hidden = false;
 
       input.value = this.view.getCellText(row, column);
 
       input.select();
-      input.inputField.focus();
+      input.focus();
 
       this._editingRow = row;
       this._editingColumn = column;
@@ -1576,7 +1651,7 @@
       this.ensureRowIsVisible(edge);
     }
 
-    _handleEnter(event) {
+    _handleEnter() {
       if (this._editingColumn) {
         this.stopEditing(true);
         this.focus();
@@ -1585,6 +1660,42 @@
 
       return this.changeOpenState(this.currentIndex);
     }
+
+    #verticalScrollbar = null;
+    #lastScrollEventTimeStampMap = new Map();
+
+    #canScroll(event) {
+      const lastScrollEventTimeStamp = this.#lastScrollEventTimeStampMap.get(
+        event.type
+      );
+      this.#lastScrollEventTimeStampMap.set(event.type, event.timeStamp);
+
+      if (
+        window.windowUtils.getWheelScrollTarget() ||
+        event.axis == event.HORIZONTAL_AXIS ||
+        (this.getAttribute("allowunderflowscroll") == "true" &&
+          this.getAttribute("hidevscroll") == "true")
+      ) {
+        return false;
+      }
+
+      if (
+        event.timeStamp - (lastScrollEventTimeStamp ?? 0) <
+        Services.prefs.getIntPref("mousewheel.scroll_series_timeout")
+      ) {
+        // If the time difference of previous event does not over the timeout,
+        // handle the event in tree as the same seies of events even if the
+        // current position is edge.
+        return true;
+      }
+
+      const curpos = Number(this.#verticalScrollbar.getAttribute("curpos"));
+      return (
+        (event.detail < 0 && 0 < curpos) ||
+        (event.detail > 0 &&
+          curpos < Number(this.#verticalScrollbar.getAttribute("maxpos")))
+      );
+    }
   }
 
   MozXULElement.implementCustomInterface(MozTree, [
